unit CarsComponentU;

interface

uses
  Classes, DsgnIntf;

type
  //These classes are components, hence inheriting from TComponent
  TCar = class(TComponent)
  private
    FSpeed: Integer;
  protected
    //Property writer routine
    procedure SetSpeed(Value: Integer); virtual;
  public
    //CVomponent constructors are polymorphic
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Drive; virtual;
    procedure Loaded; override;
  published
    //Speed will appear on Object Inspector as it is published
    property Speed: Integer read FSpeed write SetSpeed;
  end; //TCar

  TRobin = class(TCar)
  protected
    //Property writer routine
    procedure SetSpeed(Value: Integer); override;
  end; //TRobin

  TPorsche = class(TCar)
  protected
    //Property writer routine
    procedure SetSpeed(Value: Integer); override;
  public
    procedure Drive; override;
  end; //TPorsche

  //Custom component editor
  //Should really be in a separate unit, along
  //with the registration routine but never mind...

  //Check DsgnIntf.pas comments for details on this class
  //or alternatively look it up in the help
  TCarEditor = class(TComponentEditor)
  public
    procedure Edit; override;
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;


procedure Register;

implementation

uses
  SysUtils, Dialogs;

{ TCar }

constructor TCar.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  //ShowMessageFmt('A %s is born', [ClassName]);
  FSpeed := 1
end;

destructor TCar.Destroy;
begin
  //Any important tidy-up stuff
  inherited
end;

procedure TCar.Drive;
begin
  ShowMessage('I like driving in my car...')
end;

//This is called after all preoprties
//have been read from the form file
//This allows action to be taken which
//relies on other properties having been set
procedure TCar.Loaded;
begin
  inherited;
  //ShowMessageFmt('%s properties are read', [ClassName])
end;

procedure TCar.SetSpeed(Value: Integer);
begin
  FSpeed := Value;
end;

{ TRobin }

//Validate assigned speed and then pass it back to TCar.SetSpeed
procedure TRobin.SetSpeed(Value: Integer);
begin
//csDesigning tells us if we are running at design-time
//  if csDesigning in ComponentState then
//    ShowMessage('Design time!');
  if Value > 35 then
    Value := 35;
  inherited {SetSpeed(Value)}
end;

{ TPorsche }

procedure TPorsche.Drive;
begin
  ShowMessage('Vroom, roar, etc.')
end;

//Validate assigned speed and then pass it back to TCar.SetSpeed
procedure TPorsche.SetSpeed(Value: Integer);
begin
  if Value > 135 then
    Value := 135;
  inherited
end;

{ TCarEditor }

//Do verb number 1 when developer double-clicks
//a TCar descendant on the form designer
procedure TCarEditor.Edit;
begin
  ExecuteVerb(1)
end;

//User chose a custom component context menu item
//Perform appropriate action
procedure TCarEditor.ExecuteVerb(Index: Integer);
var
  CurrentSpeed: String;
begin
  case Index of
    0: ShowMessageFmt('Hello world from a %s called "%s"!',
         [Component.Classname, Component.Name]);
    1:
    begin
      //Get current speed in kph
      CurrentSpeed := IntToStr((Component as TCar).Speed * 8 div 5);
      if InputQuery('Change kph speed', 'Enter new spped (in kph):',
           CurrentSpeed) then
      begin
        //Set Speed property in mph
        (Component as TCar).Speed :=
          StrToInt(CurrentSpeed) * 5 div 8;
        //Tell form designer & Object Inspector
        //that something has changed
        Designer.Modified
      end
    end
  end
end;

//Return custom component context menu item captions
function TCarEditor.GetVerb(Index: Integer): string;
begin
  case Index of
    0: Result := '&About ' + Component.ClassName;
    1: Result := 'Set speed in kph';
  end
end;

//How mny custom component context menu items?
function TCarEditor.GetVerbCount: Integer;
begin
  Result := 2
end;

//Install this unit in a design-time package
//and Delphi will call this routine
procedure Register;
begin
  RegisterComponents('BUG', [TRobin, TPorsche]);
  RegisterComponentEditor(TCar, TCarEditor)
end;

end.
